# USRP radio with bare metal server

We use an external deployment node to control both gNB and UE. It is important to use a deployment node that is neither the UE, nor the gNB to avoid interfering with the experiment itself.

This experiment connects:
* the N320 USRP from the [R2LAB anechoic chamber](https://r2lab.inria.fr/) to the bare-metal fronthaul `sopnode-f1` node, located in the SLICES-RI SophiaNode datacenter, which serves as a gNB. The gNB is linked to the 5G core deployed in the central hub's Kubernetes cluster.
* the N300 USRP from the [R2LAB anechoic chamber](https://r2lab.inria.fr/) to the bare-metal `sopnode-f2`, located in the SLICES-RI SophiaNode datacenter, which serves as UE.

The diagram below illustrates the connections between the components.

```mermaid
graph TD
  centralhub <--> gnb
  gnb <--> n320
  ue <--> n300
  n300 <-.-> n320
  deploy -.-|controls| sopnode-f1
  deploy -.-|controls| sopnode-f2

  subgraph Slices resources

    subgraph Deployement node
    deploy[standard-2-1]
    end
    
    subgraph "centralhub k8s cluster"
            centralhub(5G core)
    end

    subgraph "Sophia Node"
        subgraph "Datacenter"
            subgraph "sopnode-f1"
                gnb(gnb)
            end
            subgraph "sopnode-f2"
                ue(UE)
            end
        end

        subgraph "Anechoic chamber"
            n320(n320)
            n300(n300)
        end
    end
  end
```

## Start gNB

To start the gNB, connect to `sopnode-f1` and provision it execute
```bash
provision.sh
```

It builds UHD and OAI and congigure the network to be able to reach the USRPs.

> **NOTE:** IP addresses are now allocated with DHCP, do not set your own

Ones the node is provisionned, start the gNB with:

```bash
./start_device.sh --usrp=31B3A77 --gnb
```

It indicates to start the gNB and connect to the USRP with serial number `31B3A77`.

Output of the gNB can be found in `~/results/gnb.{stdou|stderr}`. The PID of the gNB is saved in `~/results/gnb.pid`. When you experient is finished, just run

```bash
kill $(cat ~/results/gnb.pid)
```

to stop it.

To verify that the gNB is running properly, you can check AMF logs from the deployment node direcly checking the AMF logs in k8s with something similar to 

```bash
kubectl -n 01jkzm0ne1f0ss7p7vdg72kkve-dsaucezi-core logs -f oai-amf-6bc98ddb5c-xh2sq
```
You should see that the gNB with the name `gNB-OAI` is `Connected`.

## Start the UE

The UE is configured as follows, which is one of the default UE installed in OAI database; make sure to configure your core accordingly:


* **imsi**: 001010000000100
* **key**: fec86ba6eb707ed08905757b1bb44b8f
* **opc**: C42449363BBAD02B66D16BC975D77CC1
* **dnn**: oai
* **nssai_sst**: 1
* **nssai_sd**: 16777215


To start the UE, connect to `sopnode-f2` and provision it execute
```bash
provision.sh
```

It builds UHD and OAI and congigure the network to be able to reach the USRPs.

> **NOTE:** IP addresses are now allocated with DHCP, do not set your own

Ones the node is provisionned, start the UE with:

```bash
./start_device.sh --usrp=31D98C7 --ue
```

It indicates to start the UE and connect to the USRP with serial number `31D98C7`.

Output of the UE can be found in `~/results/ue.{stdou|stderr}`. The PID of the UE is saved in `~/results/ue.pid`. When you experient is finished, just run

```bash
kill $(cat ~/results/ue.pid)
```

to stop it.

To verify that the UE is running properly, you can check AMF logs from the deployment node direcly checking the AMF logs in k8s with something similar to 

```bash
kubectl -n 01jkzm0ne1f0ss7p7vdg72kkve-dsaucezi-core logs -f oai-amf-6bc98ddb5c-xh2sq
```
You should see that IMSI `001010000000100` is `5GMM-REGISTERED`.

To validate that the PDU session is established, on the UE machine (i.e., `sopnode-f2`) you should see the network interface `oaitun_ue1`

If you run the command 

```bash
ip addr show dev oaitun_ue1
```

you should see that you received an IP address in the ``12.1.1.0/24` prefix.
Try if you have connectivity with your UPF via ping:

```bash
ping -c 1 -i oaitun_ue1 12.1.1.1
```